##
#
# alice.tcl v1.1.0 - by strikelight ([sL] @ EFNet) (6/18/02)
#
# - Read the README.TXT for script details, help, and contact info.
#
##

#### CONFIGURATION ####

## File to save alice cookies to ##
set alice_cookiesfile "$nick.cookies"

## Every night, expire cookies that have not been accessed in how many days? ##
set alice_cookiesexpire 5

## Let your bot respond to private messages? (0 = No, 1 = Yes) ##
set alice_respondpriv 1

## Let your bot respond to comments in channel when bot's name is mentioned? ##
# (0 = No, 1 = Yes)
set alice_respondchan 1

## Require bot's nick to be in all lines of a public conversation in order
# for it to respond, after initial contact? (ie. <someone> alice: hello there)
# (if alice_respondchan is set to 1, else ignore this)
# (0 = No, 1 = Yes)
set alice_respondrequirenick 0

## Respond with user's nick? (ie. <alice> user: Hi, how are you?)
# (if alice_respondchan is set to 1, else ignore this)
# (0 = No, 1 = Yes)
set alice_respondwithnick 0

## Channels to listen on (if alice_respondchan is set to 1, else ignore this) ##
# (Separate multiple channels by space, use * to indicate all channels)
# eg. set alice_channels "#channel1 #channel2 #etc"  .. or :
set alice_channels "*"

## Also if alice_respondchan is set to 1 (otherwise ignore this),
# how long a silence should the bot wait for user's text in a channel
# after the first contact is made before giving up on their
# conversation? (in minutes)
set alice_timeoutchat 5

## Which server should I use for the alice engine?
# 0 = www.alicebot.org (default- Alice)
# 1 = www.agentruby.com (Ruby)
# 2 = ncd3.math.harvard.edu (math alice- Sofia)
# 3 = jaczone.com (Cyber-Ivar)
# 4 = your own alice server
set alice_engine -1

# If using your own alice server, enter the url here:
set alice_url "http://yourhost:2001/"

#### END OF CONFIGURATION (STOP EDITTING HERE!!) ####

if {[info exists egghttp(version)]} {
  regsub -all "\\." $egghttp(version) "" egghttpnumvers 
}
if {![info exists egghttp(version)] || ($egghttpnumvers < 102)} {
  putlog "alice.tcl : Could not load alice.tcl, egghttp.tcl 1.0.2 or greater is required."
  putlog "alice.tcl : You may download it from http://www.TCLScript.com"
  catch {unset alice_url}
  catch {unset alice_timeoutchat}
  catch {unset alice_channels}
  catch {unset alice_respondchan}
  catch {unset alice_respondrequirenick}
  catch {unset alice_respondwithnick}
  catch {unset alice_respondpriv}
  catch {unset alice_cookiesexpire}
  catch {unset alice_cookiesfile}
} elseif {$alice_engine == -1} {
  putlog "alice.tcl : You have not fully configured your Alice.tcl script."
  putlog "alice.tcl : Please edit the configuration, and then restart."
  putlog "alice.tcl : alice.tcl has NOT been loaded."
} else {
  set alice_version "1.1.0"

  switch $alice_engine {
    0 {
      set alice_url "http://alicebot.org/a.l.i.c.e./"
      set alice_input "text"
    }
    1 {
      set alice_url "http://www.agentruby.com:2001/"
      set alice_input "text"
    }
    2 {
      set alice_url "http://ncd3.math.harvard.edu:2001/"
      set alice_input "text"
    }
    3 {
      set alice_url "http://213.67.11.109/"
      set alice_input "text"
    }
    4 {
      set alice_url $alice_url
      set alice_input "text"
    }
    default {
      set alice_url "http://alicebot.org/a.l.i.c.e./"
      set alice_input "text"
    }
  }

# MISC PROCS

## strip_html : remove text that begins with <, ends with >, and everything in between
  proc strip_html {text} {
    regsub -all ">" $text \x81 text
    regsub -all "<\[^\x81\]*\x81" $text {} text
    regsub -all "&nbsp;" $text " " text
    regsub -all "&amp;" $text {\&} text
    regsub -all "&quot;" $text "\'" text
    regsub -all "&copy;" $text "(c)" text
    regsub -all "&#" $text "#" text
    regsub -all "&lt;" $text "<" text
    regsub -all "&gt;" $text ">" text
    regsub -all "%20" $text " " text
    return $text
  }

## lstring_remove : remove all elements from a list that contain the matching remove pattern
  proc lstring_remove {list remove} {
    set newlist ""
    foreach item $list {
      if {![string match "*[string tolower $remove]*" [string tolower $item]]} {
        lappend newlist $item
      }
    }
    return $newlist
  }

## nearestindex : return the string index where char occurs in text, nearest to
#                 a specified index
  proc nearestindex {text index {char " "}} {
    set tchar [string index $text $index]
    while {($tchar != $char) && ($index >= 0)} {
      incr index -1
      set tchar [string index $text $index]
    }
    return $index
  }

## istoldquiet : check to see if the text says for the bot to be quiet
  proc istoldquiet {text} {
    set exprs {shush {be *quiet} bye cya ttyl {talk* later} {i* leav} {got* go} {shut *up} shutup {leave me *alone} {go away} quiet shh}
    foreach item $exprs {
      if {[string match "*$item*" [string tolower $text]]} { return 1 }
    }
    return 0
  }

# Alice Formatting Procs

## alice:corrections : replace any occurances of the alice name with the bot's name instead
#                      also fix misc. html codes to plain text
  proc alice:corrections {text} {
    global nick
    regsub -all -- "alice" $text "$nick" text
    regsub -all -- "Alice" $text "$nick" text
    regsub -all -- "ALICE" $text "$nick" text
    regsub -all -- "A.L.I.C.E." $text "$nick" text
    regsub -all -- "A.L.I.C.E" $text "$nick" text
    regsub -all -- "a.l.i.c.e." $text "$nick" text
    regsub -all -- "a.l.i.c.e" $text "$nick" text
    regsub -all -- "ruby" $text "$nick" text
    regsub -all -- "Ruby" $text "$nick" text
    regsub -all -- "sofia" $text "$nick" text
    regsub -all -- "Sofia" $text "$nick" text
    regsub -all -- "testbot" $text "$nick" text
    regsub -all -- "TestBot" $text "$nick" text
    regsub -all -- "ivar" $text "$nick" text
    regsub -all -- "Ivar" $text "$nick" text
    regsub -all -- "[format %c 9]" $text "" text
    regsub -all -- "parent.noAnswer=1;" $text "" text
    regsub -all -- "charNoAnswer \\(\\);" $text "" text
    return $text
  }

## alice:fixquery : replace all spaces with "%20"
  proc alice:fixquery {text} {
    regsub -all -- " " $text "%20" text
    return $text
  }

# Alice Cookie Procs

## alice:getcookies : return the saved cookie for a person
#                     (cookies are used so that alice remembers the person)
  proc alice:getcookies {who} {
    global alice_cookies
    set who [string tolower $who]
    if {![info exists alice_cookies($who,data)]} {
      return ""
    }
    return $alice_cookies($who,data)
  }

## alice:setcookies : store a cookie for a person
  proc alice:setcookies {who cookies} {
    global alice_cookies
    set who [string tolower $who]
    set ocookies [alice:getcookies $who]
    if {($ocookies != "") && ([llength [split $cookies]] == 2)} {
      set ncookies ""
      foreach ck $ocookies {
        if {[string match "*jsessionid*" $ck]} {
          if {[string index $ck [expr [string length $ck] - 1]] != ";"} {
            append ck ";"
          }
          append ncookies "$ck "
        }
      }
      append ncookies $cookies
      set cookies $ncookies
    } elseif {($ocookies != "") && ([llength [split $cookies]] == 1)} {
      set ncookies ""
      foreach ck $ocookies {
        if {[string match "*alicebot_*" $ck]} {
          if {[string index $ck [expr [string length $ck] - 1]] != ";"} {
            append ck ";"
          }
          append ncookies "$ck "
        }
      }
      append ncookies $cookies
      set cookies $ncookies
    }
    set alice_cookies($who,data) "$cookies"
    set alice_cookies($who,time) "[clock seconds]"
    return 1
  }

## alice:savecookies : save cookies to file
  proc alice:savecookies {} {
    global alice_cookiesfile alice_cookies
    set outfile [open "$alice_cookiesfile" w]
    puts $outfile [list array set alice_cookies [array get alice_cookies]]
    close $outfile
    return 1
  }

## alice:loadcookies : load cookies from file
  proc alice:loadcookies {} {
    global alice_cookiesfile alice_cookies
    catch {unset alice_cookies}
    source $alice_cookiesfile
  }

## alice:expirecookies : remove old cookies
  proc alice:expirecookies {} {
    global alice_cookies alice_cookiesexpire
    set seconds [expr $alice_cookiesexpire * 24 * 60 * 60]
    foreach cookie [array names alice_cookies *,time] {
      if {[expr [clock seconds] - $alice_cookies($cookie)] >= $seconds} {
        set who [lindex [split $cookie ","] 0]
        catch {unset alice_cookies($who,time)}
        catch {unset alice_cookies($who,data)}
      }
    }
  }

## alice:time_cookie : call the expire cookies procedure
  bind time - "16 * * * *" alice:time_cookie
  proc alice:time_cookie {mi ho da mo yr} {
    putloglev o * "ALICE: Expiring old alice cookies ..."
    alice:expirecookies
    putloglev o * "ALICE: Saving alice cookie data ..."
    alice:savecookies
  }

# Alice Connection Procs

## alice:query : send user's text to the alice engine on the web to wait for a reply
  proc alice:query {who where text} {
    global alice_url alice_input
    set text [alice:fixquery $text]
    set cookies [alice:getcookies $who]
    if {$cookies != ""} {
      set sock [egghttp:geturl $alice_url [list alice:response $who $where] -query $alice_input=$text -headers "Cookie: $cookies"]
      alice:setcookies $who "$cookies"
    } else {
      set sock [egghttp:geturl $alice_url [list alice:response $who $where] -query $alice_input=$text]
    }
    return $sock
  }

## alice:response : parse alice's response and show the user
  proc alice:response {who where sock} {
    global alice_engine alice_respondwithnick
    set headers [egghttp:headers $sock]
    set data [egghttp:data $sock]
    set response ""
    if {([egghttp:errormsg $sock] != "Ok") || ($data == "")} {
      catch {alice:timeoutchat $who $where}
      egghttp:cleanup $sock
      return
    }
    egghttp:cleanup $sock
    set cookies ""
    foreach line [split $headers "\n"] {
      if {[string match "*Set-Cookie:*" $line]} {
        set what [lindex [split $line " "] 1]
        if {[string match "*Expires*" $what]} {
          set what [string range $what 0 [expr [string first "Expires" $what] - 2]]
        }
        if {[string match "*Path*" $what]} {
          set what [string range $what 0 [expr [string first "Path" $what] - 2]]
        }
        if {$what != ""} {
          append cookies "; $what"
        }
      }
    }
    set cookies [string range $cookies 2 end]
    if {$cookies != ""} {
      alice:setcookies $who "$cookies"
    }
    set a 0
    foreach line [split $data "\n"] {
      if {($a >= 1) && ![string match "*</p>*" $line]} {
        append response [string trim $line]
      }
      if {($a >= 1) && ([string match "*</p>*" $line] || [string match "*check_return()*" $line] || [string match "*</form>*" $line] || [string match "*jaczone.com*" $line])} {
        break
      }
      if {[string match "*response-line*" $line] || [string match "*\"botresponse\"*" $line] || ([string match "*offline for maint*" $line] && ($alice_engine != 3)) \
       || [string match "*</td></td></tr><tr><td align=center>*" $line] || [string match "*Perhaps that*" $line]} {
        incr a
      }
    }
    if {$response != ""} {
      set response [strip_html $response]
      set response [alice:corrections $response]
      set response [string trimleft $response "\}"]
      set response [string trim $response]
      if {[string length $response] >= 256} {
        while {($response != "") && ([string length $response] >= 256)} {
          set tmp [nearestindex "$response" 256]
          if {$tmp != -1} {
            set text [string range $response 0 [expr $tmp - 1]]
            set response [string range $response [expr $tmp + 1] end]
          } else {
            set text [string range $response 0 256]
            set response [string range $response 257 end]
          }
          if {[validchan $where] && $alice_respondwithnick} {
            set text "$who: $text"
          }
          puthelp "PRIVMSG $where :$text"
        }
      }
      if {$response != ""} {
        if {[validchan $where] && $alice_respondwithnick} {
          set response "$who: $response"
        }
        puthelp "PRIVMSG $where :$response"
      }
    }
  }

  proc alice:timeoutchat {who where} {
    global alice_chat
    catch {unset alice_chat($who,$where)}
  }

# Alice Chat Binding Procs

  if {$alice_respondpriv} {
    bind msgm - * alice:msg_query
    proc alice:msg_query {nick uhost hand text} {
      global botnick
      if {([string tolower $nick] == [string tolower $botnick]) || ([matchattr $hand b])} {
        return
      }
      # let's not bother looking at passwords and stuff ... -sL (6/13/02)
      if {[regexp pass|op|invite|ident|addhost [string tolower [lindex [split $text] 0]]]} {
        set text ""
        return 0
      }
      set where "[string tolower $nick]"
      if {($hand != "") && ($hand != "*")} {
        set who "[string tolower $hand]"
      } else {
        set who "[string tolower $nick]"
      }
      alice:query $who $where "$text"
      return 0
    }
  }

  if {$alice_respondchan} {
    bind pubm - * alice:pub_query
    proc alice:pub_query {nick uhost hand chan text} {
      global botnick alice_channels alice_timeoutchat alice_chat alice_respondrequirenick
      if {([string tolower $nick] == [string tolower $botnick]) || ([matchattr $hand b])} {
        return
      }
      set chan [string tolower $chan]
      if {($alice_channels != "*") && ([lsearch [split [string tolower $alice_channels]] $chan] == -1)} {
        return
      }
      set where "$chan"
      if {($hand != "") && ($hand != "*")} {
        set who "[string tolower $hand]"
      } else {
        set who "[string tolower $nick]"
      }
      if {(![string match "*[string tolower $botnick]*" [string tolower $text]]) && (![info exists alice_chat($who,$chan)])} {
        return
      }
      if {(![string match "*[string tolower $botnick]*" [string tolower $text]] && $alice_respondrequirenick)} {
        return
      }
      if {![info exists alice_chat($who,$chan)]} {
        set alice_chat($who,$chan) [utimer [expr $alice_timeoutchat * 60] "alice:timeoutchat $who $chan"]
      } else {
        catch {killutimer $alice_chat($who,$chan)}
        set alice_chat($who,$chan) [utimer [expr $alice_timeoutchat * 60] "alice:timeoutchat $who $chan"]
      }
      if {[istoldquiet $text]} {
        alice:timeoutchat $who $where
      }
      set text [join [lstring_remove [split $text] $botnick]]
      alice:query $who $where "$text"
    }
  }

## create cookies file if it doesn't exist
  if {![file exists $alice_cookiesfile]} {
    set outfile [open $alice_cookiesfile w]
    close $outfile
    catch {unset outfile}
  }

## load cookies file
  alice:loadcookies
  putlog "alice.tcl $alice_version by strikelight is now loaded."
}
